#ifndef DHCPMESSAGE_H
#define DHCPMESSAGE_H

/*
 * DHCP Client
 *
 * Copyright 2006 Stefan Rompf <sux@loplof.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation
 *
 */

#include <sys/types.h>
#include <netinet/in.h>

#define MAX_OPT_LEN 512


#define BOOTP_OPTION_NETMASK            1
#define BOOTP_OPTION_GATEWAY            3
#define BOOTP_OPTION_DNS                6
#define BOOTP_OPTION_HOSTNAME           12
#define BOOTP_OPTION_BOOTFILE           13
#define BOOTP_OPTION_DOMAIN             15
#define BOOTP_OPTION_BROADCAST          28
#define BOOTP_OPTION_NISDOMAIN          40

#define DHCP_OPTION_LOGSRVS             7
#define DHCP_OPTION_LPRSRVS             9
#define DHCP_OPTION_NTPSRVS             42
#define DHCP_OPTION_XFNTSRVS            48
#define DHCP_OPTION_XDMSRVS             49
#define DHCP_OPTION_REQADDR             50
#define DHCP_OPTION_LEASE               51
#define DHCP_OPTION_OVERLOAD            52
#define DHCP_OPTION_TYPE                53
#define DHCP_OPTION_SERVER              54
#define DHCP_OPTION_OPTIONREQ           55
#define DHCP_OPTION_MAXSIZE             57
#define DHCP_OPTION_T1                  58
#define DHCP_OPTION_T2                  59
#define DHCP_OPTION_CLASS_IDENTIFIER    60
#define DHCP_OPTION_CLIENT_IDENTIFIER   61

#define BOOTP_CLIENT_PORT       68
#define BOOTP_SERVER_PORT       67

#define BOOTP_OPCODE_REQUEST    1
#define BOOTP_OPCODE_REPLY      2

#define NORESPONSE              -10
#define DHCP_TYPE_DISCOVER      1
#define DHCP_TYPE_OFFER         2
#define DHCP_TYPE_REQUEST       3
#define DHCP_TYPE_DECLINE       4
#define DHCP_TYPE_ACK           5
#define DHCP_TYPE_NAK           6
#define DHCP_TYPE_RELEASE       7
#define DHCP_TYPE_INFORM        8



struct dhcp_message {
  u_int8_t *pos, *last;

  /* embedded DHCP message */
  u_int8_t op;
  u_int8_t htype;
  u_int8_t hlen;
  u_int8_t hops;
  u_int32_t xid;
  u_int16_t secs;
  u_int16_t flags;
  u_int32_t ciaddr;
  u_int32_t yiaddr;
  u_int32_t siaddr;
  u_int32_t giaddr;
  u_int8_t chaddr[16];
  u_int8_t sname[64];
  u_int8_t file[128];
  u_int8_t options[MAX_OPT_LEN];
} __attribute__((packed)) ;


void dm_init(struct dhcp_message *msg);

void dm_finish_options(struct dhcp_message *msg);

void dm_add_option(struct dhcp_message *msg, u_int8_t option,
		   u_int8_t length, void *opt);

u_int8_t *dm_next_option(struct dhcp_message *msg);

int dm_send_msg(int sock, struct in_addr *to,
		struct dhcp_message *msg);

int dm_broadcast_msg_raw(int sock, struct in_addr from_ip,
			 int from_if, struct dhcp_message *msg);

int dm_recv_msg_raw(int sock, char *from_mac,
		    struct in_addr *from_ip, struct dhcp_message *msg);

#endif

